@@ -13,9 +13,9 @@ import Folder from '@joplin/lib/models/Folder';
 import Note from '@joplin/lib/models/Note';
 import ItemList from '../gui/ItemList';
 import HelpButton from '../gui/HelpButton';
-import { surroundKeywords, nextWhitespaceIndex, removeDiacritics } from '@joplin/lib/string-utils';
+import { surroundKeywords, nextWhitespaceIndex, removeDiacritics, escapeRegExp, KeywordType } from '@joplin/lib/string-utils';
 import { mergeOverlappingIntervals } from '@joplin/lib/ArrayUtils';
-import markupLanguageUtils from '../utils/markupLanguageUtils';
+import markupLanguageUtils from '@joplin/lib/utils/markupLanguageUtils';
 import focusEditorIfEditorCommand from '@joplin/lib/services/commands/focusEditorIfEditorCommand';
 import Logger from '@joplin/utils/Logger';
 import { MarkupLanguage, MarkupToHtml } from '@joplin/renderer';
@@ -40,6 +40,39 @@ interface GotoAnythingSearchResult {
 	item_type?: ModelType;
 }
 
+// GotoAnything supports several modes:
+//
+// - Default: Search in note title, body. Can search for folders, tags, etc. This is the full
+//   featured GotoAnything.
+//
+// - TitleOnly: Search in note titles only.
+//
+// These different modes can be set from the `gotoAnything` command.
+
+export enum Mode {
+	Default = 0,
+	TitleOnly,
+}
+
+export interface UserDataCallbackEvent {
+	type: ModelType;
+	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
+	item: any;
+}
+
+export type UserDataCallbackResolve = (event: UserDataCallbackEvent)=> void;
+export type UserDataCallbackReject = (error: Error)=> void;
+export interface UserDataCallback {
+	resolve: UserDataCallbackResolve;
+	reject: UserDataCallbackReject;
+}
+
+export interface GotoAnythingUserData {
+	startString?: string;
+	mode?: Mode;
+	callback?: UserDataCallback;
+}
+
 interface Props {
 	themeId: number;
 	// eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
@@ -47,15 +80,14 @@ interface Props {
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
 	folders: any[];
 	showCompletedTodos: boolean;
-	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-	userData: any;
+	userData: GotoAnythingUserData;
 }
 
 interface State {
 	query: string;
 	results: GotoAnythingSearchResult[];
 	selectedItemId: string;
-	keywords: string[];
+	keywords: (string | ComplexTerm)[];
 	listType: number;
 	showHelp: boolean;
 	resultsInBody: boolean;
@@ -131,8 +163,8 @@ class DialogComponent extends React.PureComponent<Props, State> {
 	private itemListRef: any;
 	private listUpdateQueue_: AsyncActionQueue;
 	private markupToHtml_: MarkupToHtml;
-	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-	private userCallback_: any = null;
+	private userCallback_: UserDataCallback|null = null;
+	private mode_: Mode;
 
 	public constructor(props: Props) {
 		super(props);
@@ -142,6 +174,8 @@ class DialogComponent extends React.PureComponent<Props, State> {
 		this.userCallback_ = props?.userData?.callback;
 		this.listUpdateQueue_ = new AsyncActionQueue(100);
 
+		this.mode_ = props?.userData?.mode ? props.userData.mode : Mode.Default;
+
 		this.state = {
 			query: startString,
 			results: [],
@@ -341,6 +375,13 @@ class DialogComponent extends React.PureComponent<Props, State> {
 				// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
 				resultsInBody = !!results.find((row: any) => row.fields.includes('body'));
 
+				if (this.mode_ === Mode.TitleOnly) {
+					resultsInBody = false;
+					results = results.filter(r => {
+						return r.fields.includes('title');
+					});
+				}
+
 				const resourceIds = results.filter(r => r.item_type === ModelType.Resource).map(r => r.item_id);
 				const resources = await Resource.resourceOcrTextsByIds(resourceIds);
 
@@ -375,9 +416,15 @@ class DialogComponent extends React.PureComponent<Props, State> {
 					results = results.filter(r => !!notesById[r.id])
 						.map(r => ({ ...r, title: notesById[r.id].title }));
 
-					const normalizedKeywords = (await this.keywords(searchQuery)).map(
-						({ valueRegex }: ComplexTerm) => new RegExp(removeDiacritics(valueRegex), 'ig'),
-					);
+					const keywordRegexes = (await this.keywords(searchQuery)).map(term => {
+						if (typeof term === 'string') {
+							return new RegExp(escapeRegExp(term), 'ig');
+						} else if (term.valueRegex) {
+							return new RegExp(removeDiacritics(term.valueRegex), 'ig');
+						} else {
+							return new RegExp(escapeRegExp(term.value), 'ig');
+						}
+					});
 
 					for (let i = 0; i < results.length; i++) {
 						const row = results[i];
@@ -392,7 +439,7 @@ class DialogComponent extends React.PureComponent<Props, State> {
 								const normalizedBody = removeDiacritics(body);
 
 								// Iterate over all matches in the body for each search keyword
-								for (const keywordRegex of normalizedKeywords) {
+								for (const keywordRegex of keywordRegexes) {
 									for (const match of normalizedBody.matchAll(keywordRegex)) {
 										// Populate 'indices' with [begin index, end index] of each note fragment
 										// Begins at the regex matching index, ends at the next whitespace after seeking 15 characters to the right
@@ -525,16 +572,8 @@ class DialogComponent extends React.PureComponent<Props, State> {
 
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
 	private listItem_onClick(event: any) {
-		const itemId = event.currentTarget.getAttribute('data-id');
-		const parentId = event.currentTarget.getAttribute('data-parent-id');
-		const itemType = Number(event.currentTarget.getAttribute('data-type'));
-
-		void this.gotoItem({
-			id: itemId,
-			parent_id: parentId,
-			type: itemType,
-			commandArgs: this.state.commandArgs,
-		});
+		const targetResultId = event.currentTarget.getAttribute('id');
+		void this.gotoItem(this.selectedItem(targetResultId));
 	}
 
 	public renderItem(item: GotoAnythingSearchResult, index: number) {
@@ -546,7 +585,7 @@ class DialogComponent extends React.PureComponent<Props, State> {
 
 		const wrapKeywordMatches = (unescapedContent: string) => {
 			return surroundKeywords(
-				this.state.keywords,
+				this.state.keywords as KeywordType,
 				unescapedContent,
 				`<span class="match-highlight" style="font-weight: bold; color: ${theme.searchMarkerColor}; background-color: ${theme.searchMarkerBackgroundColor}">`,
 				'</span>',
@@ -554,9 +593,7 @@ class DialogComponent extends React.PureComponent<Props, State> {
 			);
 		};
 
-		const titleHtml = item.fragments
-			? `<span style="font-weight: bold; color: ${theme.color};">${item.title}</span>`
-			: wrapKeywordMatches(item.title);
+		const titleHtml = wrapKeywordMatches(item.title);
 
 		const fragmentsHtml = !item.fragments ? null : wrapKeywordMatches(item.fragments);
 
@@ -580,8 +617,8 @@ class DialogComponent extends React.PureComponent<Props, State> {
 				aria-posinset={index + 1}
 			>
 				<div style={style.rowTitle} dangerouslySetInnerHTML={{ __html: titleHtml }}></div>
-				{fragmentComp}
-				{pathComp}
+				{this.mode_ === Mode.TitleOnly ? null : fragmentComp}
+				{this.mode_ === Mode.TitleOnly ? null : pathComp}
 			</div>
 		);
 	}
@@ -597,8 +634,8 @@ class DialogComponent extends React.PureComponent<Props, State> {
 		return -1;
 	}
 
-	public selectedItem() {
-		const index = this.selectedItemIndex();
+	public selectedItem(itemId: string = undefined) {
+		const index = this.selectedItemIndex(undefined, itemId);
 		if (index < 0) return null;
 		return { ...this.state.results[index], commandArgs: this.state.commandArgs };
 	}
@@ -636,7 +673,9 @@ class DialogComponent extends React.PureComponent<Props, State> {
 	}
 
 	private calculateMaxHeight(itemHeight: number) {
-		const maxItemCount = Math.floor((0.7 * window.innerHeight) / itemHeight);
+		const listContainer: HTMLElement|null = this.itemListRef.current?.container;
+		const containerWindow = listContainer?.ownerDocument?.defaultView ?? window;
+		const maxItemCount = Math.floor((0.7 * containerWindow.innerHeight) / itemHeight);
 		return maxItemCount * itemHeight;
 	}
 
@@ -662,6 +701,14 @@ class DialogComponent extends React.PureComponent<Props, State> {
 		);
 	}
 
+	private helpText() {
+		if (this.mode_ === Mode.TitleOnly) {
+			return _('Type a note title to search for it.');
+		} else {
+			return _('Type a note title or part of its content to jump to it. Or type # followed by a tag name, or @ followed by a notebook name. Or type : to search for commands.');
+		}
+	}
+
 	public render() {
 		const style = this.style();
 		const helpTextId = 'goto-anything-help-text';
@@ -672,7 +719,7 @@ class DialogComponent extends React.PureComponent<Props, State> {
 				id={helpTextId}
 				style={style.help}
 				hidden={!this.state.showHelp}
-			>{_('Type a note title or part of its content to jump to it. Or type # followed by a tag name, or @ followed by a notebook name. Or type : to search for commands.')}</div>
+			>{this.helpText()}</div>
 		);
 
 		return (
