@@ -1,16 +1,25 @@
 
-export type SaveDrawingCallback = (svgElement: SVGElement, isAutosave: boolean)=> void;
+export type SaveDrawingCallback = (svgData: string, isAutosave: boolean)=> void;
 export type UpdateEditorTemplateCallback = (newTemplate: string)=> void;
+export type UpdateToolbarCallback = (toolbarData: string)=> void;
 
 export interface ImageEditorCallbacks {
-	saveDrawing: SaveDrawingCallback;
-	updateEditorTemplate: UpdateEditorTemplateCallback;
+	onLoadedEditor: ()=> void;
 
-	saveThenClose: (svgData: SVGElement)=> void;
+	save: SaveDrawingCallback;
+	updateEditorTemplate: UpdateEditorTemplateCallback;
+	updateToolbarState: UpdateToolbarCallback;
+
+	saveThenClose: (svgData: string)=> void;
 	closeEditor: (promptIfUnsaved: boolean)=> void;
 	setImageHasChanges: (hasChanges: boolean)=> void;
+
+	writeClipboardText: (text: string)=> Promise<void>;
+	readClipboardText: ()=> Promise<string>;
 }
 
+export interface ImageEditorControl {}
+
 // Overrides translations in js-draw -- as of the time of this writing,
 // Joplin has many common strings localized better than js-draw.
 export interface LocalizedStrings {
