@@ -1,4 +1,4 @@
-const React = require('react');
+import * as React from 'react';
 import { useMemo } from 'react';
 import { themeStyle } from './global-style';
 import { TextInput, TextInputProps, StyleSheet } from 'react-native';
