@@ -1,7 +1,6 @@
 import { defaultState } from '@joplin/lib/reducer';
 import { AppState } from './types';
 
-
 export const DEFAULT_ROUTE = {
 	type: 'NAV_GO',
 	routeName: 'Notes',
@@ -10,12 +9,16 @@ export const DEFAULT_ROUTE = {
 
 const appDefaultState: AppState = {
 	smartFilterId: undefined,
-	...defaultState,
+	keyboardVisible: false,
 	route: DEFAULT_ROUTE,
 	noteSelectionEnabled: false,
 	noteSideMenuOptions: null,
 	isOnMobileData: false,
 	disableSideMenuGestures: false,
 	showPanelsDialog: false,
+	...defaultState,
+
+	// On mobile, it's possible to select notes that aren't in the selected folder/tag/etc.
+	allowSelectionInOtherFolders: true,
 };
 export default appDefaultState;
