@@ -1,9 +1,8 @@
 import type { Theme } from '@joplin/lib/themes/type';
 import type { EditorEvent } from './events';
 
-// Editor commands. For compatibility, the string values of these commands
-// should correspond with the CodeMirror 5 commands:
-// https://codemirror.net/5/doc/manual.html#commands
+// Editor commands. Plugins can access these commands using editor.execCommand
+// or, in some cases, by prefixing the command name with `editor.`.
 export enum EditorCommandType {
 	Undo = 'undo',
 	Redo = 'redo',
@@ -33,6 +32,7 @@ export enum EditorCommandType {
 	InsertHorizontalRule = 'textHorizontalRule',
 
 	// Find commands
+	ToggleSearch = 'textSearch',
 	ShowSearch = 'find',
 	HideSearch = 'hideSearchDialog',
 	FindNext = 'findNext',
@@ -40,6 +40,8 @@ export enum EditorCommandType {
 	ReplaceNext = 'replace',
 	ReplaceAll = 'replaceAll',
 
+	EditLink = 'textLink',
+
 	// Editing and navigation commands
 	ScrollSelectionIntoView = 'scrollSelectionIntoView',
 	DeleteLine = 'deleteLine',
@@ -72,8 +74,9 @@ export enum EditorCommandType {
 	SelectedText = 'selectedText',
 	InsertText = 'insertText',
 	ReplaceSelection = 'replaceSelection',
-
 	SetText = 'setText',
+
+	JumpToHash = 'jumpToHash',
 }
 
 // Because the editor package can run in a WebView, plugin content scripts
@@ -90,6 +93,11 @@ export interface ContentScriptData {
 // Intended to correspond with https://codemirror.net/docs/ref/#state.Transaction%5EuserEvent
 export enum UserEventSource {
 	Paste = 'input.paste',
+	Drop = 'input.drop',
+}
+
+export interface UpdateBodyOptions {
+	noteId?: string;
 }
 
 export interface EditorControl {
@@ -106,7 +114,7 @@ export interface EditorControl {
 	setScrollPercent(fraction: number): void;
 
 	insertText(text: string, source?: UserEventSource): void;
-	updateBody(newBody: string): void;
+	updateBody(newBody: string, UpdateBodyOptions?: UpdateBodyOptions): void;
 
 	updateSettings(newSettings: EditorSettings): void;
 
@@ -139,6 +147,7 @@ export interface EditorTheme extends Theme {
 	contentMaxWidth?: number;
 	marginLeft?: number;
 	marginRight?: number;
+	listTabSize?: string;
 }
 
 export interface EditorSettings {
@@ -152,6 +161,7 @@ export interface EditorSettings {
 	useExternalSearch: boolean;
 
 	automatchBraces: boolean;
+	autocompleteMarkup: boolean;
 
 	// True if internal command keyboard shortcuts should be ignored (thus
 	// allowing Joplin shortcuts to run).
@@ -160,12 +170,16 @@ export interface EditorSettings {
 	language: EditorLanguageType;
 
 	keymap: EditorKeymap;
+	tabMovesFocus: boolean;
 
+	markdownMarkEnabled: boolean;
 	katexEnabled: boolean;
 	spellcheckEnabled: boolean;
 	readOnly: boolean;
 
 	indentWithTabs: boolean;
+
+	editorLabel: string;
 }
 
 export type LogMessageCallback = (message: string)=> void;
@@ -173,9 +187,16 @@ export type OnEventCallback = (event: EditorEvent)=> void;
 export type PasteFileCallback = (data: File)=> Promise<void>;
 type OnScrollPastBeginningCallback = ()=> void;
 
+interface Localisations {
+	[editorString: string]: string;
+}
+
 export interface EditorProps {
 	settings: EditorSettings;
 	initialText: string;
+	initialNoteId: string;
+	// Used mostly for internal editor library strings
+	localisations?: Localisations;
 
 	// If null, paste and drag-and-drop will not work for resources unless handled elsewhere.
 	onPasteFile: PasteFileCallback|null;
