@@ -13,7 +13,7 @@ import Resource from './Resource';
 import syncDebugLog from '../services/synchronizer/syncDebugLog';
 import { toFileProtocolPath, toForwardSlashes } from '../path-utils';
 const { pregQuote, substrWithEllipsis } = require('../string-utils.js');
-const { _ } = require('../locale');
+const { _, _n } = require('../locale');
 import { pull, removeElement, unique } from '../ArrayUtils';
 import { LoadOptions, SaveOptions } from './utils/types';
 import ActionLogger from '../utils/ActionLogger';
@@ -29,7 +29,7 @@ export interface PreviewsOrder {
 	dir: string;
 }
 
-interface PreviewsOptions {
+export interface PreviewsOptions {
 	order?: PreviewsOrder[];
 	conditions?: string[];
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
@@ -825,12 +825,20 @@ export default class Note extends BaseItem {
 
 		let savedNote = await super.save(o, options);
 
-		void ItemChange.add(BaseModel.TYPE_NOTE, savedNote.id, isNew ? ItemChange.TYPE_CREATE : ItemChange.TYPE_UPDATE, changeSource, beforeNoteJson);
+		void ItemChange.add(BaseModel.TYPE_NOTE, savedNote.id, isNew ? ItemChange.TYPE_CREATE : ItemChange.TYPE_UPDATE, {
+			changeSource, changeId: options?.changeId, beforeChangeItemJson: beforeNoteJson,
+		});
 
 		if (dispatchUpdateAction) {
+			// The UI requires share_id -- if a new note, it will always be the empty string in the database
+			// until processed by the share service. At present, loading savedNote from the database in this case
+			// breaks tests.
+			if (!('share_id' in savedNote) && isNew) {
+				savedNote.share_id = '';
+			}
 			// Ensures that any note added to the state has all the required
 			// properties for the UI to work.
-			if (!('deleted_time' in savedNote)) {
+			if (!('deleted_time' in savedNote) || !('share_id' in savedNote)) {
 				const fields = removeElement(unique(this.previewFields().concat(Object.keys(savedNote))), 'type_');
 				savedNote = await this.load(savedNote.id, {
 					fields,
@@ -843,6 +851,7 @@ export default class Note extends BaseItem {
 				provisional: isProvisional,
 				ignoreProvisionalFlag: ignoreProvisionalFlag,
 				changedFields: changedFields,
+				changeId: options?.changeId,
 				...options?.dispatchOptions,
 			});
 		}
@@ -884,8 +893,7 @@ export default class Note extends BaseItem {
 					'updated_time = ?',
 				];
 
-				// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-				const params: any[] = [
+				const params: (string|number)[] = [
 					now,
 					now,
 				];
@@ -898,7 +906,7 @@ export default class Note extends BaseItem {
 				const sql = `
 					UPDATE notes
 					SET	${updateSql.join(', ')}						
-					WHERE id IN ('${processIds.join('\',\'')}')
+					WHERE id IN (${this.escapeIdsForSql(processIds)})
 				`;
 
 				await this.db().exec({ sql, params });
@@ -913,7 +921,9 @@ export default class Note extends BaseItem {
 
 			for (let i = 0; i < processIds.length; i++) {
 				const id = processIds[i];
-				void ItemChange.add(BaseModel.TYPE_NOTE, id, changeType, changeSource, beforeChangeItems[id]);
+				void ItemChange.add(BaseModel.TYPE_NOTE, id, changeType, {
+					changeSource, beforeChangeItemJson: beforeChangeItems[id],
+				});
 
 				this.dispatch({
 					type: 'NOTE_DELETE',
@@ -931,7 +941,7 @@ export default class Note extends BaseItem {
 			if (!note) return null;
 			msg = _('Permanently delete note "%s"?', substrWithEllipsis(note.title, 0, 32));
 		} else {
-			msg = _('Permanently delete these %d notes?', noteIds.length);
+			msg = _n('Permanently delete this note?', 'Permanently delete these %d notes?', noteIds.length, noteIds.length);
 		}
 		return msg;
 	}
@@ -1019,6 +1029,7 @@ export default class Note extends BaseItem {
 					FROM notes
 					WHERE
 						is_conflict = 0
+						AND deleted_time = 0
 						${showCompletedTodos ? '' : 'AND todo_completed = 0'}
 					AND parent_id = ?
 				`;
