@@ -91,8 +91,7 @@ interface SettingSections {
 
 interface DefaultMigration {
 	name: string;
-	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-	previousDefault: any;
+	previousDefault: string | boolean | number;
 }
 
 // To create a default migration:
@@ -117,6 +116,10 @@ const defaultMigrations: DefaultMigration[] = [
 		name: 'themeAutoDetect',
 		previousDefault: false,
 	},
+	{
+		name: 'ocr.enabled',
+		previousDefault: false,
+	},
 ];
 
 // "UserSettingMigration" are used to migrate existing user setting to a new setting. With a way
