@@ -6,15 +6,19 @@ import Note from './models/Note';
 import Resource from './models/Resource';
 import { basename, fileExtension, safeFileExtension } from './path-utils';
 import * as fs from 'fs-extra';
-import * as pdfJsNamespace from 'pdfjs-dist';
 import { writeFile } from 'fs/promises';
 import { ResourceEntity } from './services/database/types';
-import { TextItem } from 'pdfjs-dist/types/src/display/api';
 import replaceUnsupportedCharacters from './utils/replaceUnsupportedCharacters';
 import { FetchBlobOptions } from './types';
+import { fromFile as fileTypeFromFile } from 'file-type';
+import crypto from './services/e2ee/crypto';
 
 import FileApiDriverLocal from './file-api-driver-local';
 import * as mimeUtils from './mime-utils';
+import BaseItem from './models/BaseItem';
+import { Size } from '@joplin/utils/types';
+import { cpus } from 'os';
+import type PdfJs from './utils/types/pdfJs';
 const { _ } = require('./locale');
 const http = require('http');
 const https = require('https');
@@ -106,7 +110,8 @@ interface ShimInitOptions {
 	electronBridge: any;
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
 	nodeSqlite: any;
-	pdfJs: typeof pdfJsNamespace;
+	pdfJs: PdfJs;
+	isAppleSilicon?: ()=> boolean;
 }
 
 function shimInit(options: ShimInitOptions = null) {
@@ -118,6 +123,7 @@ function shimInit(options: ShimInitOptions = null) {
 		electronBridge: null,
 		nodeSqlite: null,
 		pdfJs: null,
+		isAppleSilicon: () => false,
 		...options,
 	};
 
@@ -135,6 +141,7 @@ function shimInit(options: ShimInitOptions = null) {
 	shim.Geolocation = GeolocationNode;
 	shim.FormData = require('form-data');
 	shim.sjclModule = require('./vendor/sjcl.js');
+	shim.crypto = crypto;
 	shim.electronBridge_ = options.electronBridge;
 
 	shim.fsDriver = () => {
@@ -142,6 +149,10 @@ function shimInit(options: ShimInitOptions = null) {
 		return shim.fsDriver_;
 	};
 
+	shim.sharpEnabled = () => {
+		return !!sharp;
+	};
+
 	shim.dgram = () => {
 		return dgram;
 	};
@@ -161,6 +172,16 @@ function shimInit(options: ShimInitOptions = null) {
 		return Array.from(buffer);
 	};
 
+	shim.isAppleSilicon = () => {
+		return options.isAppleSilicon ? options.isAppleSilicon() : false;
+	};
+
+	shim.platformArch = () => {
+		const c = cpus();
+		if (!c.length) return '';
+		return c[0].model;
+	};
+
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
 	shim.detectAndSetLocale = function(Setting: any) {
 		let locale = shim.isElectron() ? shim.electronBridge().getLocale() : process.env.LANG;
@@ -266,10 +287,17 @@ function shimInit(options: ShimInitOptions = null) {
 			return await saveOriginalImage();
 		} else {
 			// For the CLI tool
-			const image = sharp(filePath);
-			const md = await image.metadata();
 
-			if (md.width <= maxDim && md.height <= maxDim) {
+			let md: Size = null;
+			// eslint-disable-next-line @typescript-eslint/no-explicit-any
+			let image: any = null;
+
+			if (sharp) {
+				image = sharp(filePath);
+				md = await image.metadata();
+			}
+
+			if (!md || (md.width <= maxDim && md.height <= maxDim)) {
 				await shim.fsDriver().copy(filePath, targetPath);
 				return true;
 			}
@@ -304,18 +332,13 @@ function shimInit(options: ShimInitOptions = null) {
 			...options,
 		};
 
-		const readChunk = require('read-chunk');
-		const imageType = require('image-type');
-
 		const isUpdate = !!options.destinationResourceId;
 
-		const uuid = require('./uuid').default;
-
 		if (!(await fs.pathExists(filePath))) throw new Error(_('Cannot access %s', filePath));
 
 		defaultProps = defaultProps ? defaultProps : {};
 
-		let resourceId = defaultProps.id ? defaultProps.id : uuid.create();
+		let resourceId = defaultProps.id ? defaultProps.id : BaseItem.generateUuid();
 		if (isUpdate) resourceId = options.destinationResourceId;
 
 		let resource = isUpdate ? {} : Resource.new();
@@ -330,11 +353,10 @@ function shimInit(options: ShimInitOptions = null) {
 		let fileExt = safeFileExtension(fileExtension(filePath));
 
 		if (!resource.mime) {
-			const buffer = await readChunk(filePath, 0, 64);
-			const detectedType = imageType(buffer);
+			const detectedType = await fileTypeFromFile(filePath);
 
 			if (detectedType) {
-				fileExt = detectedType.ext;
+				fileExt = fileExt ? fileExt : detectedType.ext;
 				resource.mime = detectedType.mime;
 			} else {
 				resource.mime = 'application/octet-stream';
@@ -381,7 +403,13 @@ function shimInit(options: ShimInitOptions = null) {
 	};
 
 	shim.attachFileToNoteBody = async function(noteBody, filePath, position = null, options = null) {
-		options = { createFileURL: false, markupLanguage: 1, ...options };
+		options = {
+			createFileURL: false,
+			markupLanguage: 1,
+			resourcePrefix: '',
+			resourceSuffix: '',
+			...options,
+		};
 
 		const { basename } = require('path');
 		const { escapeTitleText } = require('./markdownUtils').default;
@@ -402,23 +430,22 @@ function shimInit(options: ShimInitOptions = null) {
 		if (noteBody && position) newBody.push(noteBody.substr(0, position));
 
 		if (!options.createFileURL) {
-			newBody.push(Resource.markupTag(resource, options.markupLanguage));
+			newBody.push(options.resourcePrefix + Resource.markupTag(resource, options.markupLanguage) + options.resourceSuffix);
 		} else {
 			const filename = escapeTitleText(basename(filePath)); // to get same filename as standard drag and drop
 			const fileURL = `[${filename}](${toFileProtocolPath(filePath)})`;
-			newBody.push(fileURL);
+			newBody.push(options.resourcePrefix + fileURL + options.resourceSuffix);
 		}
 
 		if (noteBody) newBody.push(noteBody.substr(position));
 
-		return newBody.join('\n\n');
+		return newBody.join('');
 	};
 
-	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-	shim.attachFileToNote = async function(note, filePath, position: number = null, options: any = null) {
+	shim.attachFileToNote = async function(note, filePath, options = {}) {
 		if (!options) options = {};
 		if (note.markup_language) options.markupLanguage = note.markup_language;
-		const newBody = await shim.attachFileToNoteBody(note.body, filePath, position, options);
+		const newBody = await shim.attachFileToNoteBody(note.body, filePath, options.position ?? 0, options);
 		if (!newBody) return null;
 
 		const newNote = { ...note, body: newBody };
@@ -456,7 +483,7 @@ function shimInit(options: ShimInitOptions = null) {
 		} else {
 			throw new Error('Unsupported method');
 		}
-	},
+	};
 
 	shim.imageFromDataUrl = async function(imageDataUrl, filePath, options = null) {
 		if (options === null) options = {};
@@ -795,7 +822,7 @@ function shimInit(options: ShimInitOptions = null) {
 				const textContent = await page.getTextContent();
 
 				const strings = textContent.items.map(item => {
-					const text = (item as TextItem).str ?? '';
+					const text = item.str ?? '';
 					return text;
 				}).join('\n');
 
@@ -811,32 +838,24 @@ function shimInit(options: ShimInitOptions = null) {
 	};
 
 	shim.pdfToImages = async (pdfPath: string, outputDirectoryPath: string, options?: CreatePdfFromImagesOptions): Promise<string[]> => {
-		// We handle both the Electron app and testing framework. Potentially
-		// the same code could be use to support the CLI app.
-		const isTesting = !shim.isElectron();
+		if (typeof HTMLCanvasElement === 'undefined') {
+			throw new Error('Unsupported -- the Canvas element is required.');
+		}
 
 		const createCanvas = () => {
-			if (isTesting) {
-				return require('canvas').createCanvas();
-			}
 			return document.createElement('canvas');
 		};
 
-		// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-		const canvasToBuffer = async (canvas: any): Promise<Buffer> => {
+		const canvasToBuffer = async (canvas: HTMLCanvasElement): Promise<Buffer> => {
 			const quality = 0.8;
-			if (isTesting) {
-				return canvas.toBuffer('image/jpeg', { quality });
-			} else {
-				const canvasToBlob = async (canvas: HTMLCanvasElement): Promise<Blob> => {
-					return new Promise(resolve => {
-						canvas.toBlob(blob => resolve(blob), 'image/jpg', quality);
-					});
-				};
+			const canvasToBlob = async (canvas: HTMLCanvasElement): Promise<Blob> => {
+				return new Promise(resolve => {
+					canvas.toBlob(blob => resolve(blob), 'image/jpg', quality);
+				});
+			};
 
-				const blob = await canvasToBlob(canvas);
-				return Buffer.from(await blob.arrayBuffer());
-			}
+			const blob = await canvasToBlob(canvas);
+			return Buffer.from(await blob.arrayBuffer());
 		};
 
 		const filePrefix = `page_${Date.now()}`;
@@ -851,6 +870,9 @@ function shimInit(options: ShimInitOptions = null) {
 				const viewport = page.getViewport({ scale: options?.scaleFactor ?? 2 });
 				const canvas = createCanvas();
 				const ctx = canvas.getContext('2d');
+				if (!ctx) {
+					throw new Error('Unable to get 2D rendering context from canvas.');
+				}
 
 				canvas.height = viewport.height;
 				canvas.width = viewport.width;
