@@ -2,6 +2,8 @@ import * as React from 'react';
 import { NoteEntity, ResourceEntity } from './services/database/types';
 import type FsDriverBase from './fs-driver-base';
 import type FileApiDriverLocal from './file-api-driver-local';
+import { Crypto } from './services/e2ee/types';
+import { MarkupLanguage } from '@joplin/renderer';
 
 export interface CreateResourceFromPathOptions {
 	resizeLargeImages?: 'always' | 'never' | 'ask';
@@ -32,6 +34,26 @@ interface FetchOptions {
 	agent?: unknown;
 }
 
+interface AttachFileToNoteOptions {
+	resizeLargeImages?: 'always'|'never';
+	position?: number;
+	markupLanguage?: MarkupLanguage;
+}
+
+export enum MessageBoxType {
+	Confirm = 'question',
+	Error = 'error',
+	Info = 'info',
+}
+
+export interface ShowMessageBoxOptions {
+	title?: string;
+	buttons?: string[];
+	type?: MessageBoxType;
+	defaultId?: number;
+	cancelId?: number;
+}
+
 let isTestingEnv_ = false;
 
 // We need to ensure that there's only one instance of React being used by all
@@ -134,6 +156,12 @@ const shim = {
 		return typeof process !== 'undefined' && process.platform === 'darwin';
 	},
 
+	// Tells whether the computer **CPU** is an Apple Silicon (not whether the running version was
+	// built for ARM64)
+	isAppleSilicon: (): boolean => {
+		throw new Error('Not implemented: isAppleSilicon');
+	},
+
 	platformName: () => {
 		if (shim.isReactNative()) return shim.mobilePlatform();
 		if (shim.isMac()) return 'darwin';
@@ -144,6 +172,23 @@ const shim = {
 		throw new Error('Cannot determine platform');
 	},
 
+	// Tells the computer CPU architecture. Which if different from the architecture the running
+	// version was built for. For example, the laptop CPU may be an ARM64, while the version was
+	// built for x64 architecture. Here we want to know the laptop CPU.
+	platformArch: (): string => {
+		throw new Error('Not implemented: platformArch');
+	},
+
+	deviceString: () => {
+		const output: string[] = [];
+
+		output.push(shim.platformName());
+
+		if (shim.platformArch()) output.push(shim.platformArch());
+
+		return output.join(', ');
+	},
+
 	// "ios" or "android", or "" if not on mobile
 	mobilePlatform: () => {
 		return ''; // Default if we're not on mobile (React Native)
@@ -277,6 +322,10 @@ const shim = {
 		throw new Error('Not implemented: fsDriver');
 	},
 
+	sharpEnabled: (): boolean => {
+		return true;
+	},
+
 	FileApiDriverLocal: null as typeof FileApiDriverLocal,
 
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
@@ -292,6 +341,8 @@ const shim = {
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
 	sjclModule: null as any,
 
+	crypto: null as Crypto,
+
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
 	randomBytes: async (_count: number): Promise<any> => {
 		throw new Error('Not implemented: randomBytes');
@@ -305,8 +356,7 @@ const shim = {
 	// eslint-disable-next-line @typescript-eslint/ban-types -- Old code before rule was applied
 	detectAndSetLocale: null as Function,
 
-	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-	attachFileToNote: async (_note: any, _filePath: string): Promise<NoteEntity> => {
+	attachFileToNote: async (_note: NoteEntity, _filePath: string, _options?: AttachFileToNoteOptions): Promise<NoteEntity> => {
 		throw new Error('Not implemented: attachFileToNote');
 	},
 
@@ -388,13 +438,16 @@ const shim = {
 	// Returns the index of the button that was clicked. By default,
 	// 0 -> OK
 	// 1 -> Cancel
-	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
-	showMessageBox: (_message: string, _options: any = null): Promise<number> => {
+	showMessageBox: (_message: string, _options: ShowMessageBoxOptions = null): Promise<number> => {
 		throw new Error('Not implemented');
 	},
 
+	showErrorDialog: async (message: string): Promise<void> => {
+		await shim.showMessageBox(message, { type: MessageBoxType.Error });
+	},
+
 	showConfirmationDialog: async (message: string): Promise<boolean> => {
-		return await shim.showMessageBox(message) === 0;
+		return await shim.showMessageBox(message, { type: MessageBoxType.Confirm }) === 0;
 	},
 
 	// eslint-disable-next-line @typescript-eslint/no-explicit-any -- Old code before rule was applied
